@@ -1,6 +1,6 @@
 import redis
 
-from redisgraph.util import random_string, quote_string
+from redisgraph.util import random_string, quote_string, stringify_param_value
 from redisgraph.query_result import QueryResult
 from redisgraph.exceptions import VersionMismatchException
 
@@ -10,12 +10,20 @@ class Graph:
     Graph, collection of nodes and edges.
     """
 
-    def __init__(self, name, redis_con):
+    def __init__(self, name, redis_con=None, host='localhost', port=6379, password=None):
         """
         Create a new graph.
+
+        Args:
+            name: string that represents the name of the graph
+            redis_con: connection to Redis
         """
         self.name = name                 # Graph key
-        self.redis_con = redis_con
+        if redis_con is not None:
+            self.redis_con = redis_con
+        else:
+            self.redis_con = redis.Redis(host, port, password=password)
+
         self.nodes = {}
         self.edges = []
         self._labels = []                # List of node labels.
@@ -59,6 +67,12 @@ class Graph:
             self._properties[i] = p[0]
 
     def get_label(self, idx):
+        """
+        Returns a label by it's index
+
+        Args:
+            idx: The index of the label
+        """
         try:
             label = self._labels[idx]
         except IndexError:
@@ -68,6 +82,12 @@ class Graph:
         return label
 
     def get_relation(self, idx):
+        """
+        Returns a relationship type by it's index
+
+        Args:
+            idx: The index of the relation
+        """
         try:
             relationshipType = self._relationshipTypes[idx]
         except IndexError:
@@ -77,6 +97,12 @@ class Graph:
         return relationshipType
 
     def get_property(self, idx):
+        """
+        Returns a property by it's index
+
+        Args:
+            idx: The index of the property
+        """
         try:
             propertie = self._properties[idx]
         except IndexError:
@@ -130,24 +156,24 @@ class Graph:
         self.nodes = {}
         self.edges = []
 
-    def build_params_header(self, params):
+    def _build_params_header(self, params):
         if not isinstance(params, dict):
             raise TypeError("'params' must be a dict")
         # Header starts with "CYPHER"
         params_header = "CYPHER "
         for key, value in params.items():
-            # If value is string add quotation marks.
-            if isinstance(value, str):
-                value = quote_string(value)
-            # Value is None, replace with "null" string.
-            elif value is None:
-                value = "null"
-            params_header += str(key) + "=" + str(value) + " "
+            params_header += str(key) + "=" + stringify_param_value(value) + " "
         return params_header
 
     def query(self, q, params=None, timeout=None, read_only=False):
         """
         Executes a query against the graph.
+
+        Args:
+            q: the query
+            params: query parameters
+            timeout: maximum runtime for read queries in milliseconds
+            read_only: executes a readonly query if set to True
         """
 
         # maintain original 'q'
@@ -155,7 +181,7 @@ class Graph:
 
         # handle query parameters
         if params is not None:
-            query = self.build_params_header(params) + query
+            query = self._build_params_header(params) + query
 
         # construct query command
         # ask for compact result-set format
@@ -177,6 +203,9 @@ class Graph:
         except redis.exceptions.ResponseError as e:
             if "wrong number of arguments" in str(e):
                 print("Note: RedisGraph Python requires server version 2.2.8 or above")
+            if "unknown command" in str(e) and read_only:
+                # `GRAPH.RO_QUERY` is unavailable in older versions.
+                return self.query(q, params, timeout, read_only=False)
             raise e
         except VersionMismatchException as e:
             # client view over the graph schema is out of sync
@@ -193,11 +222,15 @@ class Graph:
         """
         Get the execution plan for given query,
         GRAPH.EXPLAIN returns an array of operations.
+
+        Args:
+            query: the query that will be executed
+            params: query parameters
         """
         if params is not None:
-            query = self.build_params_header(params) + query
+            query = self._build_params_header(params) + query
 
-        plan = self.redis_con.execute_command("GRAPH.EXPLAIN", self.name, query, query)
+        plan = self.redis_con.execute_command("GRAPH.EXPLAIN", self.name, query)
         return self._execution_plan_to_string(plan)
 
     def delete(self):
